import pandas as pd
from .config import get_param

from . import utils


def mutations(
    code_insee=None,
    in_bbox=None,
    lon_lat=None,
    fields=None,
    ordering=None,
    anneemut_min=None,
    anneemut_max=None,
    anneemut=None,
    codtypbien=None,
    idnatmut=None,
    sbati_min=None,
    sbati_max=None,
    sterr_min=None,
    sterr_max=None,
    valeurfonc_min=None,
    valeurfonc_max=None,
    vefa=None,
):
    """Retourne les mutations issues de DVF+ opendata pour le périmètre demandée sous forme d'un dataframe

    Args:
        **code_insee (str or list, optional)**: Codes INSEE communaux ou des arrondissements municipaux. Defaults to None.

        **in_bbox (list, optional)**: Emprise rectangulaire sous la forme d'une liste [longitude_min, latitude_min, longitude_min, latitude_max] - Maximum 0.02 deg x 0.02 deg. Defaults to None.

        **lon_lat (list, optional)**: Coordonnée du point au sein de la ou des mutations renvoyées [longitude, latitude]. Defaults to None.

        **fields (str, optional)**: Mettre à "all" pour obtenir tous les champs associés. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **anneemut_min (str, optional)**: Année de mutation minimale. Defaults to None.

        **anneemut_max (str, optional)**: Année de mutation maximale. Defaults to None.

        **anneemut (str, optional)**: Année de mutation. Defaults to None.

        **codtypbien (str, optional)**: Code(s) de la typologie de bien à sélectionner (il est possible de ne specifier que les premiers niveaux et de séparer par une virgule). Defaults to None.

        **idnatmut (str, optional)**: Code(s) de nature de mutation (il est possible d'en demander plusieurs en séparant par une virgule. Defaults to None.

        **sbati_min (int, optional)**: Surface batie minimale. Defaults to None.

        **sbati_max (int, optional)**: Surface batie maximale. Defaults to None.

        **sterr_min (int, optional)**: Surface de terrain minimale. Defaults to None.

        **sterr_max (int, optional)**: Surface de terrain maximale. Defaults to None.

        **valeurfonc_min (int, optional)**: Valeur foncière minimale. Defaults to None.

        **valeurfonc_max (int, optional)**: Valeur foncière maximale. Defaults to None.

        **vefa (str, optional)**: vente en l'état futur d'achevement. Defaults to None.

    Returns:
        dataframe: données sur les mutations issues de DVF+ opendata

    Examples:
        >>> import apifoncier.dvf_opendata as dvf
        >>> dvf.mutations(code_insee="59001")
        >>> dvf.mutations(in_bbox=[3, 50, 3.01, 50.01], fields="all")
        >>> dvf.mutations(code_insee=["59350", "59646"], valeurfonc_min=1000000)
    """
    result = utils.Resultat("/dvf_opendata/mutations/", **locals())
    df = result.get_dataframe()
    return df


def geomutations(
    code_insee=None,
    in_bbox=None,
    lon_lat=None,
    fields=None,
    ordering=None,
    anneemut_min=None,
    anneemut_max=None,
    anneemut=None,
    codtypbien=None,
    idnatmut=None,
    sbati_min=None,
    sbati_max=None,
    sterr_min=None,
    sterr_max=None,
    valeurfonc_min=None,
    valeurfonc_max=None,
    vefa=None,
):
    """Retourne les mutations issues de DVF+ opendata pour le périmètre demandée sous forme d'un geodataframe

    Args:
        **code_insee (str or list, optional)**: Codes INSEE communaux ou des arrondissements municipaux. Defaults to None.

        **in_bbox (list, optional)**: Emprise rectangulaire sous la forme d'une liste [longitude_min, latitude_min, longitude_min, latitude_max] - Maximum 0.02 deg x 0.02 deg. Defaults to None.

        **lon_lat (list, optional)**: Coordonnée du point au sein de la ou des mutations renvoyées [longitude, latitude]. Defaults to None.

        **fields (str, optional)**: Mettre à "all" pour obtenir tous les champs associés. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **anneemut_min (str, optional)**: Année de mutation minimale. Defaults to None.

        **anneemut_max (str, optional)**: Année de mutation maximale. Defaults to None.

        **anneemut (str, optional)**: Année de mutation. Defaults to None.

        **codtypbien (str, optional)**: Code(s) de la typologie de bien à sélectionner (il est possible de ne specifier que les premiers niveaux et de séparer par une virgule). Defaults to None.

        **idnatmut (str, optional)**: Code(s) de nature de mutation (il est possible d'en demander plusieurs en séparant par une virgule. Defaults to None.

        **sbati_min (int, optional)**: Surface batie minimale. Defaults to None.

        **sbati_max (int, optional)**: Surface batie maximale. Defaults to None.

        **sterr_min (int, optional)**: Surface de terrain minimale. Defaults to None.

        **sterr_max (int, optional)**: Surface de terrain maximale. Defaults to None.

        **valeurfonc_min (int, optional)**: Valeur foncière minimale. Defaults to None.

        **valeurfonc_max (int, optional)**: Valeur foncière maximale. Defaults to None.

        **vefa (str, optional)**: vente en l'état futur d'achevement. Defaults to None.

    Returns:
        geodataframe: données sur les mutations issues de DVF+ opendata avec les contours géométriques

    Examples:
        >>> import apifoncier.dvf_opendata as dvf
        >>> dvf.geomutations(code_insee="59001")
        >>> dvf.geomutations(in_bbox=[3, 50, 3.01, 50.01], fields="all")
        >>> dvf.geomutations(code_insee=["59350", "59646"], valeurfonc_min=1000000)
    """
    result = utils.Resultat("/dvf_opendata/geomutations/", **locals())
    gdf = result.get_geodataframe()
    return gdf


def mutation(idmutation=None):
    """Renvoi la mutation correspondant à l'identifiant idmutation (str)

    Returns:
        dataframe: donnée sur la mutation issue de DVF+ opendata
    """
    base_url = get_param("BASE_URL")
    url = f"""{base_url}/dvf_opendata/mutations/{idmutation}/"""
    response = utils.get_api_response(url)
    return pd.DataFrame.from_dict([response])



import pandas as pd
from .config import get_param

from . import utils


def aav(
    code_insee=None,
    ordering=None,
    annee=None,
    periode="annuel",
):
    """Retourne les indicateurs annuels ou triennaux de prix à l'aire d'attraction de la ville

    Args:
        **code_insee (str or list, required)**: Codes INSEE des AAV. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **annee (str, optional)**: Année. Defaults to None.

        **periode (str, optional)**: Prend la valeur "annuel" ou "triennal". Defaults to annuel.

    Returns:
        dataframe: données sur les marchés immobiliers

    Examples:
        >>> import apifoncier.ind_prix as prix
        >>> prix.aav(code_insee="001")
        >>> prix.aav(code_insee=["001", "002"], annee="2020")
        >>> prix.aav(code_insee=["001", "002"], periode="triennal")
    """
    if not periode in ("annuel", "triennal"):
        raise ValueError("Le paramètre periode doit valoir 'annuel' ou 'triennal'.")
    result = utils.Resultat(f"/indicateurs/dv3f/aav/{periode}/", **locals())
    df = result.get_dataframe(no_param_code=True)
    return df


def communes(
    code_insee=None,
    ordering=None,
    annee=None,
    periode="annuel",
):
    """Retourne les indicateurs annuels ou triennaux de prix à la commune

    Args:
        **code_insee (str or list, required)**: Codes INSEE communaux ou des arrondissements municipaux. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **annee (str, optional)**: Année. Defaults to None.

        **periode (str, optional)**: Prend la valeur "annuel" ou "triennal". Defaults to annuel.

    Returns:
        dataframe: données sur les marchés immobiliers

    Examples:
        >>> import apifoncier.ind_prix as prix
        >>> prix.communes(code_insee="59350")
        >>> prix.communes(code_insee=["59350", "59646"], annee="2020")
        >>> prix.communes(code_insee=["59350", "59646"], periode="triennal")
    """
    if not periode in ("annuel", "triennal"):
        raise ValueError("Le paramètre periode doit valoir 'annuel' ou 'triennal'.")
    result = utils.Resultat(f"/indicateurs/dv3f/communes/{periode}/", **locals())
    df = result.get_dataframe(no_param_code=True)
    return df


def departements(
    coddep=None,
    ordering=None,
    annee=None,
    periode="annuel",
):
    """Retourne les indicateurs annuels ou triennaux de prix à l'échelle du département
    Args:
        **coddep (str or list, required)**: Codes des départements. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **annee (str, optional)**: Année. Defaults to None.

        **periode (str, optional)**: Prend la valeur "annuel" ou "triennal". Defaults to annuel.

    Returns:
        dataframe: données sur les marchés immobiliers

    Examples:
        >>> import apifoncier.ind_prix as prix
        >>> prix.departements(coddep="59")
        >>> prix.departements(coddep=["59", "62"], annee="2020")
        >>> prix.departements(coddep=["59", "62"], periode="triennal")
    """
    if not periode in ("annuel", "triennal"):
        raise ValueError("Le paramètre periode doit valoir 'annuel' ou 'triennal'.")
    result = utils.Resultat(f"/indicateurs/dv3f/departements/{periode}/", **locals())
    df = result.get_dataframe(no_param_code=True)
    return df


def epci(
    code_insee=None,
    ordering=None,
    annee=None,
    periode="annuel",
):
    """Retourne les indicateurs annuels ou triennaux de prix à l'EPCI

    Args:
        **code_insee (str or list, optional)**: Codes INSEE des EPCI. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **annee (str, optional)**: Année. Defaults to None.

        **periode (str, optional)**: Prend la valeur "annuel" ou "triennal". Defaults to annuel.

    Returns:
        dataframe: données sur les marchés immobiliers

    Examples:
        >>> import apifoncier.ind_prix as prix
        >>> prix.epci(code_insee="200093201")
        >>> prix.epci(code_insee="200093201", periode="triennal")
    """
    if not periode in ("annuel", "triennal"):
        raise ValueError("Le paramètre periode doit valoir 'annuel' ou 'triennal'.")
    result = utils.Resultat(f"/indicateurs/dv3f/epci/{periode}/", **locals())
    df = result.get_dataframe(no_param_code=True)
    return df


def regions(
    code_insee=None,
    ordering=None,
    annee=None,
    periode="annuel",
):
    """Retourne les indicateurs annuels ou triennaux de prix à l'échelle de la region
    Args:
        **code_insee (str or list, required)**: Codes des regions. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **annee (str, optional)**: Année. Defaults to None.

        **periode (str, optional)**: Prend la valeur "annuel" ou "triennal". Defaults to annuel.

    Returns:
        dataframe: données sur les marchés immobiliers

    Examples:
        >>> import apifoncier.ind_prix as prix
        >>> prix.regions(code_insee="32")
        >>> prix.regions(code_insee=["32", "11"], annee="2020")
        >>> prix.regions(code_insee=["32", "11"], periode="triennal")
    """
    if not periode in ("annuel", "triennal"):
        raise ValueError("Le paramètre periode doit valoir 'annuel' ou 'triennal'.")
    result = utils.Resultat(f"/indicateurs/dv3f/regions/{periode}/", **locals())
    df = result.get_dataframe(no_param_code=True)
    return df



from . import utils


def communes(
    code_insee=None,
    ordering=None,
    annee=None,
    annee_min=None,
    annee_max=None,
):
    """Retourne les indicateurs annuels de consommation d'espace communaux

    Args:
        **code_insee (str or list, required)**: Codes INSEE communaux ou des arrondissements municipaux. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **annee_min (str, optional)**: Année minimale. Defaults to None.

        **annee_max(str, optional)**: Année maximale. Defaults to None.

        **annee (str, optional)**: Année. Defaults to None.

    Returns:
        dataframe: données sur la consomation d'espace

    Examples :
        >>> from apifoncier.ind_conso_espace as conso_enaf
        >>> conso_enaf.communes(code_insee="59350")
        >>> conso_enaf.communes(code_insee=["59350", "62041"], annee_min="2015")
    """

    result = utils.Resultat("/indicateurs/conso_espace/communes/", **locals())
    df = result.get_dataframe(no_param_code=True)
    return df


def departements(
    coddep=None,
    ordering=None,
    annee=None,
    annee_min=None,
    annee_max=None,
):
    """Retourne les indicateurs annuels de consommation d'espace departementaux

    Args:
        **coddep (str or list, required)**: Codes INSEE departementaux. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **annee_min (str, optional)**: Année minimale. Defaults to None.

        **annee_max(str, optional)**: Année maximale. Defaults to None.

        **annee (str, optional)**: Année. Defaults to None.

    Returns:
        dataframe: données sur la consomation d'espace

    Examples :
        >>> from apifoncier.ind_conso_espace as conso_enaf
        >>> conso_enaf.departements(coddep="59")
        >>> conso_enaf.departements(coddep=["59", "62"], annee_max="2015")
    """
    result = utils.Resultat("/indicateurs/conso_espace/departements/", **locals())
    df = result.get_dataframe(no_param_code=True)
    return df



import pandas as pd
from .config import get_param

from . import utils


def friches(
    code_insee=None,
    coddep=None,
    in_bbox=None,
    lon_lat=None,
    fields=None,
    ordering=None,
    surface_max=None,
    surface_min=None,
    urba_zone_type=None,
):
    """Retourne les friches issues de Cartofriches pour le périmètre demandé sous forme d'un dataframe

    Args:
        **code_insee (str or list, optional)**: Codes INSEE communaux ou des arrondissements municipaux. Defaults to None.

        **coddep (str or list, optional)**: Codes INSEE des départements. Defaults to None.

        **in_bbox (list, optional)**: Emprise rectangulaire sous la forme d'une liste [longitude_min, latitude_min, longitude_min, latitude_max]. Defaults to None.

        **lon_lat (list, optional)**: Coordonnée du point au sein de la ou des friches renvoyées [longitude, latitude]. Defaults to None.

        **fields (str, optional)**: Mettre à "all" pour obtenir tous les champs associés. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **surface_max (int, optional)**: Surface maximale des friches. Defaults to None.

        **surface_min (int, optional)**: Surface minimale des friches. Defaults to None.

        **urba_zone_type (str, optional)**: Type de zonage. Defaults to None.

    Returns:
        dataframe: données sur les friches issues de Cartofriches

    Examples:
        >>> import apifoncier.cartofriches as cartofriches
        >>> cartofriches.friches(code_insee="59350")
        >>> cartofriches.friches(coddep="59")
        >>> cartofriches.friches(in_bbox=[3, 50, 4, 51])
    """
    result = utils.Resultat("/cartofriches/friches/", **locals())
    df = result.get_dataframe()
    return df


def geofriches(
    code_insee=None,
    coddep=None,
    in_bbox=None,
    lon_lat=None,
    fields=None,
    ordering=None,
    surface_max=None,
    surface_min=None,
    urba_zone_type=None,
):
    """Retourne les friches issues de Cartofriches pour le périmètre demandé sous forme d'un geodataframe

    Args:
        **code_insee (str or list, optional)**: Codes INSEE communaux ou des arrondissements municipaux. Defaults to None.

        **coddep (str or list, optional)**: Codes INSEE des départements. Defaults to None.

        **in_bbox (list, optional)**: Emprise rectangulaire sous la forme d'une liste [longitude_min, latitude_min, longitude_min, latitude_max]. Defaults to None.

        **lon_lat (list, optional)**: Coordonnée du point au sein de la ou des friches renvoyées [longitude, latitude]. Defaults to None.

        **fields (str, optional)**: Mettre à "all" pour obtenir tous les champs associés. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **surface_max (int, optional)**: Surface maximale des friches. Defaults to None.

        **surface_min (int, optional)**: Surface minimale des friches. Defaults to None.

        **urba_zone_type (str, optional)**: Type de zonage. Defaults to None.

    Returns:
        geodataframe: données sur les friches issues de Cartofriches avec les contours géométriques

    Examples:
        >>> import apifoncier.cartofriches as cartofriches
        >>> cartofriches.geofriches(code_insee="59350")
        >>> cartofriches.geofriches(coddep="59")
        >>> cartofriches.geofriches(in_bbox=[3, 50, 4, 51])
    """
    result = utils.Resultat("/cartofriches/geofriches/", **locals())
    gdf = result.get_geodataframe()
    return gdf


def friche(site_id):
    """Renvoi la friche correpondant au site_id (str)

    Args:
        **site_id (str, required)**: Identifiant du site.

    Returns:
        dataframe: donnée sur la friche issue de Cartofriches
    """

    base_url = get_param("BASE_URL")
    url = f"""{base_url}/cartofriches/friches/{site_id}/"""
    response = utils.get_api_response(url)
    return pd.DataFrame.from_dict([response])



import pandas as pd
from .config import get_param

from . import utils


########################################################################
# PARCELLES
########################################################################


def parcelles(
    code_insee=None,
    in_bbox=None,
    lon_lat=None,
    fields=None,
    ordering=None,
    catpro3=None,
    ctpdl=None,
    dcntarti_min=None,
    dcntarti_max=None,
    dcntnaf_min=None,
    dcntnaf_max=None,
    dcntpa_min=None,
    dcntpa_max=None,
    idcomtxt=None,
    idpar=None,
    jannathmin_min=None,
    jannathmin_max=None,
    nlocal_min=None,
    nlocal_max=None,
    nlogh_min=None,
    nlogh_max=None,
    slocal_min=None,
    slocal_max=None,
    sprincp_min=None,
    sprincp_max=None,
    stoth_min=None,
    stoth_max=None,
):
    """Retourne les parcelles issues des Fichiers fonciers pour le périmètre demandé sous forme d'un dataframe

    Args:
        **code_insee (str or list, optional)**: Codes INSEE communaux ou des arrondissements municipaux. Defaults to None.

        **coddep (str or list, optional)**: Codes INSEE des départements. Defaults to None.

        **in_bbox (list, optional)**: Emprise rectangulaire sous la forme d'une liste [longitude_min, latitude_min, longitude_min, latitude_max] - Maximum 0.02 deg x 0.02 deg. Defaults to None.

        **lon_lat (list, optional)**: Coordonnée du point au sein de la ou des parcelles renvoyées [longitude, latitude]. Defaults to None.

        **fields (str, optional)**: Mettre à "all" pour obtenir tous les champs associés. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **catpro3 (str, optional)**: Chaîne(s) de caractères contenue dans le code de catégorie de
          propriétaire (il est possible de ne specifier que les premiers niveaux et
          de séparer par une virgule). Defaults to None.

        **ctpdl (str, optional)**: Type de pdl (type de copropriété). Defaults to None.

        **dcntarti_min (int, optional)**: Surface artificialisée minimale de la parcelle (m2). Defaults to None.

        **dcntarti_max (int, optional)**: Surface artificialisée maximale de la parcelle (m2). Defaults to None.

        **dcntnaf_min (int, optional)**: Surface NAF minimale de la parcelle (m2). Defaults to None.

        **dcntnaf_max (int, optional)**: Surface NAF maximale de la parcelle (m2). Defaults to None.

        **dcntpa_min (int, optional)**: Surface minimale de la parcelle (m2). Defaults to None.

        **dcntpa_max (int, optional)**: Surface maximale de la parcelle (m2). Defaults to None.

        **idcomtxt (str, optional)**: Chaine de caractères contenue dans le libellé de la commune. Defaults to None.

        **idpar (str, optional)**: Identifiants de parcelle (séparés par une virgule). Defaults to None.

        **jannathmin_min (int, optional)**: Année minimale de construction du local le plus ancien. Defaults to None.

        **jannathmin_max (int, optional)**: Année maximale de construction du local le plus ancien. Defaults to None.

        **nlocal_min (int, optional)**: Nombre de locaux minimal sur la parcelle. Defaults to None.

        **nlocal_max (int, optional)**: Nombre de locaux maximal sur la parcelle. Defaults to None.

        **nlogh_min (int, optional)**: Nombre de logements minimal sur la parcelle. Defaults to None.

        **nlogh_max (int, optional)**:Nombre de logements maximal sur la parcelle. Defaults to None.

        **slocal_min (int, optional)**: Surface minimale des parties d'évaluation (m2). Defaults to None.

        **slocal_max (int, optional)**: Surface maximale des parties d'évaluation (m2). Defaults to None.

        **sprincp_min (int, optional)**:Surface minimale des pièces principales professionnelles (m2). Defaults to None.

        **sprincp_max (int, optional)**: Surface maximale des pièces principales professionnelles (m2). Defaults to None.

        **stoth_min (int, optional)**:Surface minimale des pièces d'habitation (m2). Defaults to None.

        **stoth_max (int, optional)**: Surface maximale des pièces d'habitation (m2). Defaults to None.

    Returns:
        dataframe: données sur les parcelles issues des Fichiers fonciers

    Examples:
        >>> import apifoncier.ff as ff
        >>> ff.parcelles(code_insee="59350", dcntpa_min=3000)
        >>> ff.parcelles(in_bbox=[3,50,3.01,50.01])
    """
    result = utils.Resultat("/ff/parcelles/", use_token=True, **locals())
    df = result.get_dataframe()
    return df


def geoparcelles(
    code_insee=None,
    in_bbox=None,
    lon_lat=None,
    fields=None,
    ordering=None,
    catpro3=None,
    ctpdl=None,
    dcntarti_min=None,
    dcntarti_max=None,
    dcntnaf_min=None,
    dcntnaf_max=None,
    dcntpa_min=None,
    dcntpa_max=None,
    idcomtxt=None,
    idpar=None,
    jannathmin_min=None,
    jannathmin_max=None,
    nlocal_min=None,
    nlocal_max=None,
    nlogh_min=None,
    nlogh_max=None,
    slocal_min=None,
    slocal_max=None,
    sprincp_min=None,
    sprincp_max=None,
    stoth_min=None,
    stoth_max=None,
):
    """Retourne les parcelles issues des Fichiers fonciers pour le périmètre demandé sous forme d'un geodataframe integrant les contours géométriques

    Args:
        **code_insee (str or list, optional)**: Codes INSEE communaux ou des arrondissements municipaux. Defaults to None.

        **coddep (str or list, optional)**: Codes INSEE des départements. Defaults to None.

        **in_bbox (list, optional)**: Emprise rectangulaire sous la forme d'une liste [longitude_min, latitude_min, longitude_min, latitude_max] - Maximum 0.02 deg x 0.02 deg. Defaults to None.

        **lon_lat (list, optional)**: Coordonnée du point au sein de la ou des parcelles renvoyées [longitude, latitude]. Defaults to None.

        **fields (str, optional)**: Mettre à "all" pour obtenir tous les champs associés. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **catpro3 (str, optional)**: Chaîne(s) de caractères contenue dans le code de catégorie de
          propriétaire (il est possible de ne specifier que les premiers niveaux et
          de séparer par une virgule). Defaults to None.

        **ctpdl (str, optional)**: Type de pdl (type de copropriété). Defaults to None.

        **dcntarti_min (int, optional)**: Surface artificialisée minimale de la parcelle (m2). Defaults to None.

        **dcntarti_max (int, optional)**: Surface artificialisée maximale de la parcelle (m2). Defaults to None.

        **dcntnaf_min (int, optional)**: Surface NAF minimale de la parcelle (m2). Defaults to None.

        **dcntnaf_max (int, optional)**: Surface NAF maximale de la parcelle (m2). Defaults to None.

        **dcntpa_min (int, optional)**: Surface minimale de la parcelle (m2). Defaults to None.

        **dcntpa_max (int, optional)**: Surface maximale de la parcelle (m2). Defaults to None.

        **idcomtxt (str, optional)**: Chaine de caractères contenue dans le libellé de la commune. Defaults to None.

        **idpar (str, optional)**: Identifiants de parcelle (séparés par une virgule). Defaults to None.

        **jannathmin_min (int, optional)**: Année minimale de construction du local le plus ancien. Defaults to None.

        **jannathmin_max (int, optional)**: Année maximale de construction du local le plus ancien. Defaults to None.

        **nlocal_min (int, optional)**: Nombre de locaux minimal sur la parcelle. Defaults to None.

        **nlocal_max (int, optional)**: Nombre de locaux maximal sur la parcelle. Defaults to None.

        **nlogh_min (int, optional)**: Nombre de logements minimal sur la parcelle. Defaults to None.

        **nlogh_max (int, optional)**:Nombre de logements maximal sur la parcelle. Defaults to None.

        **slocal_min (int, optional)**: Surface minimale des parties d'évaluation (m2). Defaults to None.

        **slocal_max (int, optional)**: Surface maximale des parties d'évaluation (m2). Defaults to None.

        **sprincp_min (int, optional)**:Surface minimale des pièces principales professionnelles (m2). Defaults to None.

        **sprincp_max (int, optional)**: Surface maximale des pièces principales professionnelles (m2). Defaults to None.

        **stoth_min (int, optional)**:Surface minimale des pièces d'habitation (m2). Defaults to None.

        **stoth_max (int, optional)**: Surface maximale des pièces d'habitation (m2). Defaults to None.

    Returns:
        geodataframe: données sur les parcelles issues des Fichiers fonciers

    Examples:
        >>> import apifoncier.ff as ff
        >>> ff.geoparcelles(code_insee="59350", dcntpa_min=3000)
        >>> ff.geoparcelles(in_bbox=[3,50,3.01,50.01])
    """
    result = utils.Resultat("/ff/geoparcelles/", use_token=True, **locals())
    gdf = result.get_geodataframe()
    return gdf


def parcelle(idpar=None):
    """Renvoie la parcelle correspondant à l'identifiant idpar (str)

    Returns:
        dataframe: donnée sur la parcelle issue des Fichiers fonciers
    """
    base_url = get_param("BASE_URL")
    url = f"""{base_url}/ff/parcelles/{idpar}/"""
    response = utils.get_api_response(url, use_token=True)
    return pd.DataFrame.from_dict([response])


########################################################################
# TUPS
########################################################################


def tups(
    code_insee=None,
    in_bbox=None,
    lon_lat=None,
    fields=None,
    ordering=None,
    catpro3=None,
    idtup=None,
    typetup=None,
):
    """Retourne les tup issues des Fichiers fonciers pour le périmètre demandé sous forme d'un dataframe

    Args:
        **code_insee (str or list, optional)**: Codes INSEE communaux ou des arrondissements municipaux. Defaults to None.

        **coddep (str or list, optional)**: Codes INSEE des départements. Defaults to None.

        **in_bbox (list, optional)**: Emprise rectangulaire sous la forme d'une liste [longitude_min, latitude_min, longitude_min, latitude_max] - Maximum 0.02 deg x 0.02 deg. Defaults to None.

        **lon_lat (list, optional)**: Coordonnée du point au sein de la ou des tup renvoyées [longitude, latitude]. Defaults to None.

        **fields (str, optional)**: Mettre à "all" pour obtenir tous les champs associés. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **catpro3 (str, optional)**: Chaîne(s) de caractères contenue dans le code de catégorie de
          propriétaire (il est possible de ne specifier que les premiers niveaux et
          de séparer par une virgule). Defaults to None.

        **idtup (str, optional)**: Identifiants de tup (séparés par une virgule). Defaults to None.

        **typetup (str, optional)**: Type de pdl (type de copropriété). Defaults to None.


    Returns:
        dataframe: données sur les tup issues des Fichiers fonciers

    Examples:
        >>> import apifoncier.ff as ff
        >>> ff.tups(code_insee="59350", catpro3='P')
        >>> ff.tups(in_bbox=[3,50,3.01,50.01])
    """
    result = utils.Resultat("/ff/tups/", use_token=True, **locals())
    df = result.get_dataframe()
    return df


def geotups(
    code_insee=None,
    in_bbox=None,
    lon_lat=None,
    fields=None,
    ordering=None,
    catpro3=None,
    idtup=None,
    typetup=None,
):
    """Retourne les tup issues des Fichiers fonciers pour le périmètre demandé sous forme d'un geodataframe integrant les contours géométriques

    Args:
        **code_insee (str or list, optional)**: Codes INSEE communaux ou des arrondissements municipaux. Defaults to None.

        **coddep (str or list, optional)**: Codes INSEE des départements. Defaults to None.

        **in_bbox (list, optional)**: Emprise rectangulaire sous la forme d'une liste [longitude_min, latitude_min, longitude_min, latitude_max] - Maximum 0.02 deg x 0.02 deg. Defaults to None.

        **lon_lat (list, optional)**: Coordonnée du point au sein de la ou des tup renvoyées [longitude, latitude]. Defaults to None.

        **fields (str, optional)**: Mettre à "all" pour obtenir tous les champs associés. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **catpro3 (str, optional)**: Chaîne(s) de caractères contenue dans le code de catégorie de
          propriétaire (il est possible de ne specifier que les premiers niveaux et
          de séparer par une virgule). Defaults to None.

        **idtup (str, optional)**: Identifiants de tup (séparés par une virgule). Defaults to None.

        **typetup (str, optional)**: Type de pdl (type de copropriété). Defaults to None.


    Returns:
        geodataframe: données sur les tup issues des Fichiers fonciers

    Examples:
        >>> import apifoncier.ff as ff
        >>> ff.geotups(code_insee="59350", catpro3='P')
        >>> ff.geotups(in_bbox=[3,50,3.01,50.01])
    """
    result = utils.Resultat("/ff/geotups/", use_token=True, **locals())
    gdf = result.get_geodataframe()
    return gdf


def tup(idtup=None):
    """Renvoie la tup correspondante à l'identifiant idtup (str)

    Returns:
        dataframe: donnée sur la tup issue des Fichiers fonciers
    """
    base_url = get_param("BASE_URL")
    url = f"""{base_url}/ff/parcelles/{idtup}/"""
    response = utils.get_api_response(url, use_token=True)
    return pd.DataFrame.from_dict([response])


########################################################################
# LOCAUX
########################################################################


def locaux(
    code_insee=None,
    fields=None,
    ordering=None,
    catpro3=None,
    dteloc=None,
    idpar=None,
    idprocpte=None,
    idsec=None,
    locprop=None,
    loghlls=None,
    proba_rprs=None,
    slocal_min=None,
    slocal_max=None,
    typeact=None,
):
    """Retourne les locaux des Fichiers fonciers pour le périmètre demandé sous forme d'un dataframe

    Args:
        **code_insee (str or list, optional)**: Codes INSEE communaux ou des arrondissements municipaux. Defaults to None.

        **fields (str, optional)**: Mettre à "all" pour obtenir tous les champs associés. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **catpro3 (str or list, optional)**: Chaîne(s) de caractères contenue dans le code de catégorie de
          propriétaire (il est possible de ne specifier que les premiers niveaux et
          de séparer par une virgule). Defaults to None.

        **dteloc (str or list, optional)**: Type(s) de local (il est possible de spécifier plusieurs types
          et de séparer par une virgule). Defaults to None.

        **idpar (str, optional)**: Identifiant de parcelle. Defaults to None.

        **idprocpte (str, optional)**: Identifiant de compte communal. Defaults to None.

        **idsec (str, optional)**: Identifiant de section cadastrale_. Defaults to None.

        **locprop (str_, optional)**:Localisation généralisée du propriétaire recevant la Taxe Foncière. Defaults to None.

        **loghlls (str, optional)**: Logement d’habitation de type logement social repéré par exonération. Defaults to None.

        **proba_rprs (str, optional)**: Probabilité de résidence principale ou secondaire (il est possible
          de spécifier plusieurs types et de séparer par une virgule). Defaults to None.

        **slocal_min (int, optional)**: Surface minimale des parties d'évaluation (m2). Defaults to None.

        **slocal_max (int, optional)**: Surface maximale des parties d'évaluation (m2). Defaults to None.

        **typeact (str, optional)**: Chaîne(s) de caractères contenue dans le classement du local
          selon le type d''activité (Code catégorie du local d’activité) (il est possible
          de ne specifier que les premiers niveaux et de séparer par une virgule). Defaults to None.

    Returns:
        dataframe: données sur les locaux issues des Fichiers fonciers
    """
    result = utils.Resultat("/ff/locaux/", use_token=True, **locals())
    df = result.get_dataframe()
    return df


def local(idlocal=None):
    """Renvoie le local correspondant à l'identifiant idlocal (str)

    Returns:
        dataframe: donnée sur le local issu des Fichiers fonciers
    """
    base_url = get_param("BASE_URL")
    url = f"""{base_url}/ff/locaux/{idlocal}/"""
    response = utils.get_api_response(url, use_token=True)
    return pd.DataFrame.from_dict([response])


########################################################################
# PROPRIOS
########################################################################


def proprios(
    code_insee=None,
    fields=None,
    ordering=None,
    catpro3=None,
    ccodro=None,
    gtoper=None,
    idprocpte=None,
    locprop=None,
    typedroit=None,
):
    """Retourne les droits de propriétés des Fichiers fonciers pour le périmètre demandé sous forme d'un dataframe

    Args:
        **code_insee (str or list, optional)**: Codes INSEE communaux ou des arrondissements municipaux. Defaults to None.

        **fields (str, optional)**: Mettre à "all" pour obtenir tous les champs associés. Defaults to None.

        **ordering (str, optional)**: Champs à utiliser pour ordonner le résultat. Defaults to None.

        **catpro3 (str, optional)**: Chaîne(s) de caractères contenue dans le code de catégorie de
          propriétaire (il est possible de ne specifier que les premiers niveaux et
          de séparer par une virgule). Defaults to None.

        **ccodro (str, optional)**: Code(s) du droit réel ou particulier (il est possible de spécifier
          plusieurs valeurs et de séparer par une virgule). Defaults to None.

        **gtoper (str, optional)**: Indicateur de personne physique ou moral. Defaults to None.

        **idprocpte (str, optional)**: Identifiant de compte communal. Defaults to None.

        **locprop (str, optional)**:Localisation généralisée du propriétaire recevant la Taxe Foncière. Defaults to None.

        **typedroit (str, optional)**: Type de droit : propriétaire ou gestionnaire. Defaults to None.

    Returns:
        dataframe: données sur les locaux issues des Fichiers fonciers
    """
    result = utils.Resultat("/ff/proprios/", use_token=True, **locals())
    df = result.get_dataframe()
    return df


def proprio(idprodroit=None):
    """Renvoie le droit de propriété correspondant à l'identifiant idprodroit (str)

    Returns:
        dataframe: donnée sur le droit de propriété issu des Fichiers fonciers
    """
    base_url = get_param("BASE_URL")
    url = f"""{base_url}/ff/proprios/{idprodroit}/"""
    response = utils.get_api_response(url, use_token=True)
    return pd.DataFrame.from_dict([response])



from apifoncier.config import configure
import apifoncier.cartofriches
import apifoncier.dv3f
import apifoncier.dvf_opendata
import apifoncier.ff
import apifoncier.ind_conso_espace
import apifoncier.ind_prix



class ApiDFError(Exception):
    def __init__(self, status_code, detail):
        self.status_code = status_code
        message = f"""Erreur {status_code} : {detail}"""
        super().__init__(message)


class TokenNotConfigured(Exception):
    def __init__(self):
        message = f"""Le token n'a pas été configuré. Utiliser apifoncier.configure(TOKEN="jeton")"""
        super().__init__(message)



CONFIG_INIT = {
    "BASE_URL": "https://apidf-preprod.cerema.fr",
    "TOKEN": None,
    "PROXY": None,
    "PROGRESS_BAR": True,
    "MAX_ATTEMPTS": 3,
}
CONFIG = CONFIG_INIT.copy()


def configure(**kwargs):
    global CONFIG
    CONFIG.update(kwargs)


def get_param(value):
    global CONFIG
    return CONFIG.get(value)


def reset():
    global CONFIG
    CONFIG = CONFIG_INIT.copy()



from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="apifoncier",
    version="0.0.25",
    description="Mobiliser les données foncières de l'api du Cerema directement avec python",
    packages=find_packages(exclude=["tests"]),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/rcadot/py.apifoncier",
    author="Romain Cadot",
    author_email="romain.cadot@cerema.fr",
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords=["api", "foncier", "cerema", "dv3f", "friches", "artificialisation"],
    install_requires=["pandas", "requests", "geopandas", "tqdm"],
    extras_require={"dev": ["pytest"]},
    python_requires=">=3.9",
)



import warnings
import requests
import pandas as pd
import geopandas as gpd
from urllib.parse import urljoin

from tqdm import tqdm

from .config import get_param

from .exceptions import ApiDFError
from .exceptions import TokenNotConfigured

########################################################################
# GESTION DES RESULTATS
########################################################################


class Resultat:
    def __init__(self, url_endpoint, **kwargs):
        self.use_token = kwargs.pop("use_token", False)
        self.base_url = get_param("BASE_URL")
        self.url = urljoin(self.base_url, url_endpoint)
        self.process_params(**kwargs)

    def process_params(self, **kwargs):
        geo_params = Resultat.process_filter_params(
            lon_lat=kwargs.pop("lon_lat", None),
            in_bbox=kwargs.pop("in_bbox", None),
            code_insee=kwargs.pop("code_insee", None),
            coddep=kwargs.pop("coddep", None),
        )

        (
            self.lon_lat,
            self.in_bbox,
            self.code_insee,
            self.coddep,
        ) = Resultat.process_geo_params(**geo_params)
        self.params = Resultat.process_filter_params(**kwargs)
        ### On transforme les paramètres de type list en chaine de caractère concaténée
        for keyword, value in self.params.items():
            if isinstance(value, list):
                self.params[keyword] = ",".join([str(e).strip() for e in value])

    def get_dataframe(self, no_param_code=False):
        if self.lon_lat:
            lon = self.lon_lat[0]
            lat = self.lon_lat[1]
            lon_min = lon - 0.01
            lon_max = lon + 0.01
            lat_min = lat - 0.01
            lat_max = lat + 0.01
            in_bbox = f"""in_bbox={lon_min},{lat_min},{lon_max},{lat_max}"""
            url = f"""{self.url}?{in_bbox}&contains_geom={{"type": "Point", "coordinates":[{lon}, {lat}]}}"""
            return get_all_data(url, self.params, use_token=self.use_token)
        if self.in_bbox:
            url = f"""{self.url}?in_bbox={",".join(self.in_bbox)}&page_size=500"""
            return get_all_data(url, self.params, use_token=self.use_token)
        if self.code_insee:
            datas = []
            for code in self.code_insee:
                if not no_param_code:
                    url = f"""{self.url}?code_insee={code}&page_size=500"""
                else:
                    url = f"""{self.url}{code}/"""
                data = get_all_data(url, self.params, use_token=self.use_token)
                datas.append(data)
        if self.coddep:
            datas = []
            for code in self.coddep:
                if not no_param_code:
                    url = f"""{self.url}?coddep={code}&page_size=500"""
                else:
                    url = f"""{self.url}{code}/"""
                data = get_all_data(url, self.params, use_token=self.use_token)
                datas.append(data)
        data = datas[0] if len(datas) == 1 else pd.concat(datas, ignore_index=True)
        return data

    def get_geodataframe(self):
        if self.lon_lat:
            lon = self.lon_lat[0]
            lat = self.lon_lat[1]
            lon_min = lon - 0.01
            lon_max = lon + 0.01
            lat_min = lat - 0.01
            lat_max = lat + 0.01
            in_bbox = f"""in_bbox={lon_min},{lat_min},{lon_max},{lat_max}"""
            url = f"""{self.url}?{in_bbox}&contains_geom={{"type": "Point", "coordinates":[{lon}, {lat}]}}"""
            return get_all_geodata(url, self.params, use_token=self.use_token)
        if self.in_bbox:
            url = f"""{self.url}?in_bbox={",".join(self.in_bbox)}&page_size=500"""
            return get_all_geodata(url, self.params, use_token=self.use_token)
        if self.code_insee:
            datas = []
            for code in self.code_insee:
                url = f"""{self.url}?code_insee={code}&page_size=500"""
                data = get_all_geodata(url, self.params, use_token=self.use_token)
                datas.append(data)
        if self.coddep:
            datas = []
            for code in self.coddep:
                url = f"""{self.url}?coddep={code}&page_size=500"""
                data = get_all_geodata(url, self.params, use_token=self.use_token)
                datas.append(data)
        data = (
            datas[0]
            if len(datas) == 1
            else gpd.GeoDataFrame(pd.concat(datas, ignore_index=True))
        )
        return data

    @staticmethod
    def process_filter_params(**kwargs):
        params = {}
        for kw, value in kwargs.items():
            if value is not None:
                params[kw] = value
        return params

    @staticmethod
    def process_geo_params(**kwargs):
        keyword_priority = ["lon_lat", "in_bbox", "code_insee", "coddep"]
        # Vérification de l'obligation de préciser au moins un paramètre
        if not any(keyword in kwargs for keyword in keyword_priority):
            raise ValueError(
                "Veuillez préciser au moins un paramètre parmi code_insee, in_bbox, lonlat et coddep."
            )

        # Vérification si plusieurs mots-clés ont été utilisés
        used_keywords = [
            keyword for keyword, value in kwargs.items() if value is not None
        ]
        for keyword in keyword_priority:
            if keyword in used_keywords:
                first_keyword = keyword
                break
        if len(used_keywords) > 1:
            warning_message = f"Les mots-clés {', '.join(used_keywords)} ont été précisés. Seul le mot-clé {first_keyword} sera utilisé."
            warnings.warn(warning_message, UserWarning)

        values = {keyword: None for keyword in keyword_priority}
        # Vérification et traitement des paramètres selon la priorité des mots-clés
        for keyword in keyword_priority:
            if keyword in kwargs:
                value = kwargs[keyword]

                if keyword == "lon_lat":
                    # Vérification que lon_lat est une liste de 2 floats
                    if (
                        not isinstance(value, list)
                        or len(value) != 2
                        or not all(is_num(x) for x in value)
                    ):
                        raise ValueError(
                            "Le paramètre lon_lat doit être une liste de 2 floats."
                        )
                    values[keyword] = value
                    break
                if keyword == "in_bbox":
                    # Vérification que in_bbox est une liste de 4 floats
                    if (
                        not isinstance(value, list)
                        or len(value) != 4
                        or not all(is_num(x) for x in value)
                    ):
                        raise ValueError(
                            "Le paramètre in_bbox doit être une liste de 4 floats."
                        )
                    values[keyword] = [str(elt) for elt in value]
                    break
                elif keyword == "code_insee" or keyword == "coddep":
                    if isinstance(value, str):
                        value = [value]
                    values[keyword] = value
                    break

        return tuple(values[keyword] for keyword in keyword_priority)


########################################################################
# INTERROGATIONS API
########################################################################


def get_all_geodata(url, params=None, use_token=False):
    progress_bar = get_param("PROGRESS_BAR")
    data_pages = []
    has_more_pages = True

    pbar = None
    while has_more_pages:
        response = get_api_response(url, params, use_token)
        if len(response["features"]) > 0:
            data_page = gpd.GeoDataFrame.from_features(response).set_index(
                pd.json_normalize(response["features"])["id"].values
            )  ## On ajout les identifiants en index
            data_pages.append(data_page)
            if progress_bar:
                if not pbar:
                    pbar = tqdm(total=response["count"])
                    pbar.set_description(url)
                pbar.update(len(response["features"]))

        if not response["next"]:
            has_more_pages = False
        else:
            url = response["next"]

    if data_pages:
        data = gpd.GeoDataFrame(
            pd.concat(
                data_pages,
            )
        )  # ignore_index=True))
        return data
    else:
        return gpd.GeoDataFrame()


def get_all_data(url, params=None, use_token=False):
    progress_bar = get_param("PROGRESS_BAR")
    data_pages = []
    has_more_pages = True

    pbar = None
    while has_more_pages:
        response = get_api_response(url, params, use_token)
        if len(response["results"]) > 0:
            data_page = pd.DataFrame(response["results"])
            data_pages.append(data_page)
            if progress_bar:
                if not pbar:
                    pbar = tqdm(total=response["count"])
                    pbar.set_description(url)
                pbar.update(len(response["results"]))

        if not response["next"]:
            has_more_pages = False
        else:
            url = response["next"]

    if data_pages:
        data = pd.concat(data_pages)
        return data
    else:
        return pd.DataFrame()


def get_api_response(url, params=None, use_token=False, attempt=1):
    max_attempts = get_param("MAX_ATTEMPTS")
    HEADERS = {
        "Content-Type": "application/json",
    }

    PROXIES = None
    proxy = get_param("PROXY")
    if proxy:
        PROXIES = {"http": proxy, "https": proxy}

    if use_token:
        token = get_param("TOKEN")
        if not token:
            raise TokenNotConfigured()
        HEADERS["Authorization"] = "Token " + token

    try:
        response = requests.get(url, params=params, headers=HEADERS, proxies=PROXIES)
    except Exception as e:
        if attempt < max_attempts:
            return get_api_response(
                url, params=params, use_token=use_token, attempt=attempt + 1
            )
        else:
            raise e

    status_code = response.status_code
    if status_code != 200:
        raise ApiDFError(status_code, response.json()["detail"])

    return response.json()


def is_num(value):
    return isinstance(value, int) or isinstance(value, float)
